/*
Copyright (C) 2024 Centro de Computacao Cientifica e Software Livre
Departamento de Informatica - Universidade Federal do Parana - C3SL/UFPR

This file is part of simcaq-node.

simcaq-node is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

simcaq-node is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with simcaq-node.  If not, see <https://www.gnu.org/licenses/>.
*/

const express = require('express');

const rateSchoolNewApp = express.Router();

const libs = `${process.cwd()}/libs`;

const query = require(`${libs}/middlewares/query`).query;

const response = require(`${libs}/middlewares/response`);

const ReqQueryFields = require(`${libs}/middlewares/reqQueryFields`);

const multiQuery = require(`${libs}/middlewares/multiQuery`);

const id2str = require(`${libs}/middlewares/id2str`);

const config = require(`${libs}/config`); 

const cache = require('apicache').options({ debug: config.debug, statusCodes: {include: [200]} }).middleware;

let rqf = new ReqQueryFields();

rateSchoolNewApp.use(cache('15 day'));

rateSchoolNewApp.get('/years', (req, res, next) => {
    req.sql.from('pnad_novo')
    .field('DISTINCT pnad_novo.ano_ref', 'year')
    next();
}, query, response('years'));


rqf.addField({
    name: 'filter',
    field: false,
    where: true
}).addField({
    name: 'dims',
    field: true,
    where: false
}).addValue({
    name: 'id',
    table: 'pnad_novo',
    tableField: 'id',
    where: {
        relation: '=',
        type: 'integer',
        field: 'id'
    }
}).addValue({
    name: 'state',
    table: 'estado',
    tableField: ['id', 'nome'],
    resultField: ['state_id', 'state_nome'],
    where: {
        relation: '=',
        type: 'integer',
        field: 'id',
    },
    join: {
        primary: 'id',
        foreign: 'cod_uf',
        foreignTable: 'pnad_novo'
    }
}).addValue({
    name: 'state_not',
    table: 'estado',
    tableField: ['nome', 'id'],
    resultField: ['state_name', 'state_id'],
    where: {
        relation: '<>',
        type: 'integer',
        field: 'cod_uf',
        table: 'pnad_novo'
    },
    join: {
        primary: 'id',
        foreign: 'cod_uf',
        foreignTable: 'pnad_novo'
    }
}).addValue({
    name: 'attends_school',
    table: 'pnad_novo',
    tableField: 'frequenta_escola',
    resultField: 'attends_school_id',
    where: {
        relation: '=',
        type: 'integer',
        field: 'frequenta_escola'
    }
}).addValue({
    name: 'bolsa_familia',
    table: 'pnad_novo',
    tableField: 'recebeu_rendimentos_de_programa_bolsa_familia',
    resultField: 'bolsa_familia_id',
    where: {
        relation: '=',
        type: 'integer',
        field: 'recebeu_rendimentos_de_programa_bolsa_familia'
    }
}).addValue({
    name: 'new_pnad_ethnic_group',
    table: 'pnad_novo',
    tableField: 'cor_raca',
    resultField: 'new_pnad_ethnic_group_id',
    where: {
        relation: '=',
        type: 'integer',
        field: 'cor_raca'
    }
}).addValue({
    name: 'age_range_all',
    table: 'pnad_novo',
    tableField: 'faixa_etaria',
    resultField: 'age_range_all_id',
    where: {
        relation: '=',
        type: 'integer',
        field: 'faixa_etaria'
    }
}).addValue({
    name: 'gender',
    table: 'pnad_novo',
    tableField: 'sexo',
    resultField: 'gender_id',
    where: {
        relation: '=',
        type: 'integer',
        field: 'sexo'
    }
}).addValue({
    name: 'cap_code',
    table: 'pnad_novo',
    tableField: 'cod_cap',
    resultField: 'cap_code_id',
    where: {
        relation: '=',
        type: 'integer',
        field: 'cod_cap'
    }
}).addValue({
    name: 'region',
    table: 'pnad_novo',
    tableField: 'cod_regiao',
    resultField: 'region_id',
    where: {
        relation: '=',
        type: 'integer',
        field: 'cod_regiao'
    }
}).addValue({
    name: 'metro_code',
    table: 'pnad_novo',
    tableField: 'cod_rm_ride',
    resultField: 'metro_code_id',
    where: {
        relation: '=',
        type: 'integer',
        field: 'cod_rm_ride'
    }
}).addValue({
    name: 'min_year',
    table: 'pnad_novo',
    tableField: 'ano_ref',
    resultField: 'year',
    where: {
        relation: '>=',
        type: 'integer',
        field: 'ano_ref'
    }
}).addValue({
    name: 'max_year',
    table: 'pnad_novo',
    tableField: '',
    resultField: 'year',
    where: {
        relation: '<=',
        type: 'integer',
        field: 'ano_ref'
    }
}).addField({
    name: 'search',
    field: false,
    where: true
}).addValueToField({
    name: 'name',
    table: 'pnad_novo',
    tableField: 'nome',
    where: {
        relation: 'LIKE',
        type: 'string',
        field: 'nome'
    }
}, 'search').addValue({
    name: 'mesoregion',
    table: 'pnad_novo',
    tableField: 'mesorregiao_id',
    where: {
        relation: '=',
        type: 'integer',
        field: 'mesorregiao_id'
    }
}).addValue({
    name: 'microregion',
    table: 'pnad_novo',
    tableField: 'microrregiao_id',
    where: {
        relation: '=',
        type: 'integer',
        field: 'microrregiao_id'
    }
});

function matchQueries(attendsSchoolObj, populationObj) {
    let match = [];

    for (let i = 0; i < attendsSchoolObj.length; i++) {
        let newObj = attendsSchoolObj[i];

        newObj.total = (attendsSchoolObj[i].total / populationObj[i].total) * 100;
        //newObj.faixa_etaria = attendsSchoolObj[i].faixa_etaria;
        //newObj.year = attendsSchoolObj[i].year;

        match.push(newObj);
    }

    return match;
}

rateSchoolNewApp.get('/', rqf.parse(), rqf.build(),  (req, res, next) => {
    req.querySet = []
    
    let attends_school = req.sql.clone();
    attends_school.from('pnad_novo')
    .field('round(sum(pnad_novo.peso_domicilio_pessoas_com_cal), 0)', 'total')
    .field('pnad_novo.faixa_etaria')
    .field('pnad_novo.ano_ref', 'year')
    .where('pnad_novo.ano_ref >= 2019 AND frequenta_escola = 1')
    .where('pnad_novo.faixa_etaria < 7')
    .group('pnad_novo.ano_ref')
    .group('pnad_novo.faixa_etaria')
    .order('pnad_novo.faixa_etaria')
    .order('pnad_novo.ano_ref')
    req.querySet.push(attends_school);

    let full_population = req.sql.clone();
    full_population.from('pnad_novo')
    .field('round(sum(pnad_novo.peso_domicilio_pessoas_com_cal), 0)', 'total')
    .field('pnad_novo.faixa_etaria')
    .field('pnad_novo.ano_ref', 'year')
    .where('pnad_novo.ano_ref >= 2019')
    .where('pnad_novo.faixa_etaria < 7')
    .group('pnad_novo.ano_ref')
    .group('pnad_novo.faixa_etaria')
    .order('pnad_novo.faixa_etaria')
    .order('pnad_novo.ano_ref')
    req.querySet.push(full_population);

    next();
}, multiQuery, (req, res, next) => {
    
    let newObj = matchQueries(req.result[0], req.result[1]);
    req.result = newObj;

    next();
}, id2str.transform(false), response('rateSchoolNew'));

module.exports = rateSchoolNewApp;
